@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2012 Nikita Karetnikov <nikita@karetnikov.org>
+;;; Copyright © 2019 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -26,15 +27,16 @@
 (define-public less
   (package
     (name "less")
-    (version "487")
+    (version "551")
     (source
      (origin
-      (method url-fetch)
-      (uri (string-append "mirror://gnu/less/less-"
-                          version ".tar.gz"))
-      (sha256
-       (base32
-        "01i7n6jaxwmww3pasy3hg38zc6x7jw0w05mqqvh6caqbrdaq9p7k"))))
+       (method url-fetch)
+       (uri (list (string-append "mirror://gnu/less/less-"
+                                 version ".tar.gz")
+                  (string-append "http://www.greenwoodsoftware.com/less/less-"
+                                 version ".tar.gz")))
+       (sha256
+        (base32 "0ggyjl3yzn7c450zk1rixi9ls6asdhgqynhk34zsd0ckhmsm45pz"))))
     (build-system gnu-build-system)
     (inputs `(("ncurses" ,ncurses)))
     (home-page "https://www.gnu.org/software/less/")
