@@ -3,6 +3,7 @@
 ;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2016 Nicolas Goaziou <mail@nicolasgoaziou.fr>
 ;;; Copyright © 2016 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2018 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -28,6 +29,7 @@
   #:use-module (gnu packages audio)
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages base)
+  #:use-module (gnu packages compression)
   #:use-module (gnu packages fontutils)
   #:use-module (gnu packages gl)
   #:use-module (gnu packages glib)
@@ -36,8 +38,7 @@
   #:use-module (gnu packages linux)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages pulseaudio)
-  #:use-module (gnu packages xorg)
-  #:use-module (gnu packages zip))
+  #:use-module (gnu packages xorg))
 
 (define-public smalltalk
   (package
@@ -60,14 +61,15 @@
     (inputs
      `(("zip" ,zip)))
     (arguments
-     `(#:phases (alist-cons-before
-                 'configure 'fix-libc
-                 (lambda _
-                   (let ((libc (assoc-ref %build-inputs "libc")))
-                     (substitute* "libc.la.in"
-                       (("@LIBC_SO_NAME@") "libc.so")
-                       (("@LIBC_SO_DIR@")  (string-append libc "/lib")))))
-                %standard-phases)))
+     `(#:phases
+       (modify-phases %standard-phases
+         (add-before 'configure 'fix-libc
+           (lambda _
+             (let ((libc (assoc-ref %build-inputs "libc")))
+               (substitute* "libc.la.in"
+                 (("@LIBC_SO_NAME@") "libc.so")
+                 (("@LIBC_SO_DIR@")  (string-append libc "/lib"))))
+             #t)))))
     (home-page "http://smalltalk.gnu.org/")
     (synopsis "Smalltalk environment")
     (description
@@ -99,7 +101,8 @@ such as ones for networking and GUI programming.")
               "ux_version = \"GNU\";\n"))
            (substitute* "unix/vm/config.cmake"
              (("\\(VM_BUILD_STRING.*")
-              "(VM_BUILD_STRING \\\"Built with GNU Guix\\\")"))))))
+              "(VM_BUILD_STRING \\\"Built with GNU Guix\\\")"))
+           #t))))
     (inputs
      `(("alsa-lib" ,alsa-lib)
        ("dbus" ,dbus)
@@ -130,14 +133,15 @@ such as ones for networking and GUI programming.")
            (lambda* (#:key outputs #:allow-other-keys)
              (let ((out (assoc-ref outputs "out")))
                (with-directory-excursion "bld"
-                 (zero?
-                  (system* "../unix/cmake/configure"
+                 (invoke "../unix/cmake/configure"
                            (string-append "--prefix=" out)
-                           "--without-quartz"))))))
+                           "--without-quartz")
+                 #t))))
          (replace 'build
            (lambda _
              (with-directory-excursion "bld"
-               (zero? (system* "make"))))))))
+               (invoke "make"))
+             #t)))))
     (synopsis "Smalltalk programming language and environment")
     (description "Squeak is a full-featured implementation of the Smalltalk
 programming language and environment based on (and largely compatible with)
