@@ -2,11 +2,12 @@
 ;;; Copyright © 2013 Cyril Roelandt <tipecaml@gmail.com>
 ;;; Copyright © 2015 Amirouche Boubekki <amirouche@hypermove.net>
 ;;; Copyright © 2016 Al McElrath <hello@yrns.org>
-;;; Copyright © 2016, 2017 ng0 <ng0@infotropique.org>
+;;; Copyright © 2016, 2017 ng0 <ng0@n0.is>
 ;;; Copyright © 2015 Dmitry Bogatov <KAction@gnu.org>
 ;;; Copyright © 2015 Leo Famulari <leo@famulari.name>
 ;;; Copyright © 2016 Eric Bavier <bavier@member.fsf.org>
 ;;; Copyright © 2017 Alex Griffin <a@ajgrf.com>
+;;; Copyright © 2018, 2019 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -50,14 +51,14 @@
 (define-public blind
   (package
     (name "blind")
-    (version "1.0")
+    (version "1.1")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://dl.suckless.org/tools/blind-"
+              (uri (string-append "https://dl.suckless.org/tools/blind-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "1b36k8fg2gmabm69jckqja49i8y4rcbccgvv2wija15ciszrm1x9"))))
+                "0nncvzyipvkkd7zlgzwbjygp82frzs2hvbnk71gxf671np607y94"))))
     (build-system gnu-build-system)
     (arguments
      '(#:tests? #f                      ; no check target
@@ -68,7 +69,7 @@
        (modify-phases %standard-phases
          (delete 'configure))))         ; no configure script
     (synopsis "Command line video editing utilities")
-    (home-page "http://tools.suckless.org/blind/")
+    (home-page "https://tools.suckless.org/blind/")
     (description
      "Blind is a collection of command line video editing utilities.  It uses
 a custom raw video format with a simple container.")
@@ -77,13 +78,13 @@ a custom raw video format with a simple container.")
 (define-public dwm
   (package
     (name "dwm")
-    (version "6.1")
+    (version "6.2")
     (source (origin
              (method url-fetch)
-             (uri (string-append "http://dl.suckless.org/dwm/dwm-"
+             (uri (string-append "https://dl.suckless.org/dwm/dwm-"
                                  version ".tar.gz"))
              (sha256
-              (base32 "1zkmwb6df6m254shx06ly90c0q4jl70skk1pvkixpb7hcxhwbxn2"))))
+              (base32 "03hirnj8saxnsfqiszwl2ds7p0avg20izv9vdqyambks00p2x44p"))))
     (build-system gnu-build-system)
     (arguments
      `(#:tests? #f
@@ -99,9 +100,8 @@ a custom raw video format with a simple container.")
         (replace 'install
           (lambda* (#:key outputs #:allow-other-keys)
             (let ((out (assoc-ref outputs "out")))
-              (zero?
-               (system* "make" "install"
-                        (string-append "DESTDIR=" out) "PREFIX=")))))
+              (invoke "make" "install"
+                      (string-append "DESTDIR=" out) "PREFIX="))))
         (add-after 'build 'install-xsession
           (lambda* (#:key outputs #:allow-other-keys)
             ;; Add a .desktop file to xsessions.
@@ -126,7 +126,7 @@ a custom raw video format with a simple container.")
        ("libx11" ,libx11)
        ("libxft" ,libxft)
        ("libxinerama" ,libxinerama)))
-    (home-page "http://dwm.suckless.org/")
+    (home-page "https://dwm.suckless.org/")
     (synopsis "Dynamic window manager")
     (description
      "dwm is a dynamic window manager for X.  It manages windows in tiled,
@@ -137,14 +137,14 @@ optimising the environment for the application in use and the task performed.")
 (define-public dmenu
   (package
     (name "dmenu")
-    (version "4.7")
+    (version "4.9")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://dl.suckless.org/tools/dmenu-"
+              (uri (string-append "https://dl.suckless.org/tools/dmenu-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "1fgph549i0yw62x02jhfc8qmqk2rnzcnm46bxq6jig1cvkw3amm7"))))
+                "0ia9nqr83bv6x247q30bal0v42chcj9qcjgv59xs6xj46m7iz5xk"))))
     (build-system gnu-build-system)
     (arguments
      '(#:tests? #f ; no tests
@@ -160,7 +160,7 @@ optimising the environment for the application in use and the task performed.")
        ("libxft" ,libxft)
        ("libx11" ,libx11)
        ("libxinerama" ,libxinerama)))
-    (home-page "http://tools.suckless.org/dmenu/")
+    (home-page "https://tools.suckless.org/dmenu/")
     (synopsis "Dynamic menu")
     (description
      "A dynamic menu for X, originally designed for dwm.  It manages large
@@ -170,7 +170,7 @@ numbers of user-defined menu items efficiently.")
 (define-public spoon
   (package
     (name "spoon")
-    (version "0.3")
+    (version "0.6")
     (source
      (origin
        (method url-fetch)
@@ -178,16 +178,16 @@ numbers of user-defined menu items efficiently.")
                            name "-" version ".tar.gz"))
        (sha256
         (base32
-         "10c5i7ykpy7inzzfiw1dh0srpkljycr3blxhvd8160wsvplbws48"))))
+         "1jpmg9k9f4f3lpz0k3cphqjswlyf8lz2sm8ccifiip93kd4rrdj0"))))
     (build-system gnu-build-system)
     (arguments
-     `(#:tests? #f ; No tests
+     `(#:tests? #f                      ; no tests
        #:make-flags (list "CC=gcc"
                           (string-append "PREFIX=" %output))))
     (inputs
      `(("libx11" ,libx11)
        ("libxkbfile" ,libxkbfile)
-       ("alsa-lib" ,alsa-lib)
+       ("alsa-lib" ,alsa-lib)           ; tinyalsa (unpackaged) would suffice
        ("libmpdclient" ,libmpdclient)))
     (home-page "https://git.2f30.org/spoon/")
     (synopsis "Set dwm status")
@@ -201,7 +201,7 @@ numbers of user-defined menu items efficiently.")
     (version "1.4")
     (source (origin
               (method url-fetch)
-              (uri (string-append "http://dl.suckless.org/tools/slock-"
+              (uri (string-append "https://dl.suckless.org/tools/slock-"
                                   version ".tar.gz"))
               (sha256
                (base32
@@ -217,7 +217,7 @@ numbers of user-defined menu items efficiently.")
        ("libxext" ,libxext)
        ("libxinerama" ,libxinerama)
        ("libxrandr" ,libxrandr)))
-    (home-page "http://tools.suckless.org/slock/")
+    (home-page "https://tools.suckless.org/slock/")
     (synopsis "Simple X session lock")
     (description
      "Simple X session lock with trivial feedback on password entry.")
@@ -226,35 +226,35 @@ numbers of user-defined menu items efficiently.")
 (define-public st
   (package
     (name "st")
-    (version "0.7")
+    (version "0.8.2")
     (source
      (origin
        (method url-fetch)
-       (uri (string-append "http://dl.suckless.org/st/st-"
+       (uri (string-append "https://dl.suckless.org/st/st-"
                            version ".tar.gz"))
        (sha256
-        (base32
-         "00309qiw20rc89696pk8bdr7ik4r1aarik7jxqk8k66cdj80v1zp"))))
+        (base32 "0ddz2mdp1c7q67rd5vrvws9r0493ln0mlqyc3d73dv8im884xdxf"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:tests? #f ; no tests
+     '(#:tests? #f                      ; no tests
        #:make-flags (list "CC=gcc"
                           (string-append "PREFIX=" %output))
        #:phases
        (modify-phases %standard-phases
          (delete 'configure)
          (add-after 'unpack 'inhibit-terminfo-install
-                    (lambda _
-                      (substitute* "Makefile"
-                        (("\t@tic -s st.info") ""))
-                      #t)))))
+           (lambda _
+             (substitute* "Makefile"
+               (("\ttic .*") ""))
+             #t)))))
     (inputs
      `(("libx11" ,libx11)
        ("libxft" ,libxft)
        ("fontconfig" ,fontconfig)
        ("freetype" ,freetype)))
-    (native-inputs `(("pkg-config" ,pkg-config)))
-    (home-page "http://st.suckless.org/")
+    (native-inputs
+     `(("pkg-config" ,pkg-config)))
+    (home-page "https://st.suckless.org/")
     (synopsis "Simple terminal emulator")
     (description
      "St implements a simple and lightweight terminal emulator.  It
@@ -270,7 +270,7 @@ drawing.")
     (source
      (origin
        (method url-fetch)
-       (uri (string-append "http://dl.suckless.org/surf/surf-"
+       (uri (string-append "https://dl.suckless.org/surf/surf-"
                            version ".tar.gz"))
        (sha256
         (base32
@@ -298,7 +298,7 @@ drawing.")
        ("xprop" ,xprop)))
     (native-inputs
      `(("pkg-config" ,pkg-config)))
-    (home-page "http://surf.suckless.org/")
+    (home-page "https://surf.suckless.org/")
     (synopsis "Simple web browser")
     (description
      "Surf is a simple web browser based on WebKit/GTK+.  It is able to
@@ -310,19 +310,19 @@ point surf to another URI by setting its XProperties.")
 (define-public sent
   (package
     (name "sent")
-    (version "0.2")
+    (version "1")
     (source (origin
-              (method url-fetch)
-              (uri (string-append "http://dl.suckless.org/tools/sent-"
+              (method url-fetch/tarbomb)
+              (uri (string-append "https://dl.suckless.org/tools/sent-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "0xhh752hwaa26k4q6wvrb9jnpbnylss2aw6z11j7l9rav7wn3fak"))))
+                "0cxysz5lp25mgww73jl0mgip68x7iyvialyzdbriyaff269xxwvv"))))
     (build-system gnu-build-system)
     (arguments
      `(#:phases (modify-phases %standard-phases
-                  (delete 'configure))  ;no configuration
-       #:tests? #f                      ;no test suite
+                  (delete 'configure))  ; no configuration
+       #:tests? #f                      ; no test suite
        #:make-flags (let ((pkg-config (lambda (flag)
                                         (string-append
                                          "$(shell pkg-config " flag " "
@@ -339,19 +339,19 @@ point surf to another URI by setting its XProperties.")
        ("libx11" ,libx11)
        ("libxft" ,libxft)
        ("fontconfig" ,fontconfig)))
-    (synopsis "Plaintext presentation tool")
-    (description "Sent uses plaintext files and PNG images to create slideshow
+    (synopsis "Plain-text presentation tool")
+    (description "Sent uses plain-text files and PNG images to create slideshow
 presentations.  Each paragraph represents a slide in the presentation.
 Especially for presentations using the Takahashi method this is very nice and
 allows you to write down the presentation for a quick lightning talk within a
 few minutes.")
-    (home-page "http://tools.suckless.org/sent")
+    (home-page "https://tools.suckless.org/sent")
     (license license:x11)))
 
 (define-public xbattmon
   (package
     (name "xbattmon")
-    (version "0.9")
+    (version "1.1")
     (source
      (origin
        (method url-fetch)
@@ -359,7 +359,7 @@ few minutes.")
                            name "-" version ".tar.gz"))
        (sha256
         (base32
-         "0n2rrjq03pgqrdkl7cz5snsfdanf4s58w9h6dbvnl7p8bbd3j2kn"))))
+         "1zr6y8lml9xkx0a3dbbsds2qz1bjxvskp7wsckkf8mlsqrbb3xsg"))))
     (build-system gnu-build-system)
     (arguments
      `(#:tests? #f ; No tests
@@ -496,23 +496,22 @@ cups server to be installed.")
 (define-public noice
   (package
     (name "noice")
-    (version "0.6")
+    (version "0.8")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://dl.2f30.org/releases/"
-                           name "-" version ".tar.gz"))
+                           "noice-" version ".tar.gz"))
        (sha256
-        (base32
-         "0ldkbb71z6k4yzj4kpg3s94ijj1c1kx9dfcjz393py09scfyg5hr"))))
+        (base32 "0g01iwzscdv27c1idv93gd74kjzy3n9kazgm6qz08rygp96qx4xw"))))
     (build-system gnu-build-system)
     (arguments
-     `(#:tests? #f ; No tests
+     `(#:tests? #f                      ; no tests
        #:make-flags (list "CC=gcc"
                           (string-append "PREFIX=" %output))
        #:phases
        (modify-phases %standard-phases
-         (delete 'configure) ; No configure script
+         (delete 'configure)            ; no configure script
          (add-before 'build 'curses
            (lambda _
              (substitute* "Makefile"
@@ -525,31 +524,28 @@ cups server to be installed.")
      "Noice is a small curses-based file browser.")
     (license license:bsd-2)))
 
-;;; We want some commits that are more recent than the latest release, 0.2
 (define-public human
-  (let ((commit "50c80e6ba12823184b6866e06b955dbd2ccdc5d7")
-        (revision "1"))
-    (package
-      (name "human")
-      (version (string-append "0.2-" revision "." (string-take commit 7)))
-      (source
-       (origin
-         (method git-fetch)
-         (uri (git-reference
-               (url "git://git.2f30.org/human.git")
-               (commit commit)))
-         (file-name (string-append name "-" version "-checkout"))
-         (sha256
-          (base32
-           "18xngm4h9vsyip52zwd79rrp1irzg6rs462lpbp61amf7hj955gn"))))
+  (package
+    (name "human")
+    (version "0.3")
+    (source
+     (origin
+       (method git-fetch)
+       (uri (git-reference
+             (url "git://git.2f30.org/human.git")
+             (commit version)))
+       (file-name (git-file-name name version))
+       (sha256
+        (base32
+         "0y0bsmvpwfwb2lwspi6a799y34h1faxc6yfanyw6hygxc8661mga"))))
     (build-system gnu-build-system)
     (arguments
-     `(#:tests? #f ; No tests
+     `(#:tests? #f                      ; no tests
        #:make-flags (list "CC=gcc"
                           (string-append "PREFIX=" %output))
        #:phases
        (modify-phases %standard-phases
-         (delete 'configure)))) ; No configure script
+         (delete 'configure))))         ; no configure script
     (home-page "https://git.2f30.org/human/")
     (synopsis "Convert bytes to human readable formats")
     (description
@@ -558,12 +554,12 @@ human readable format.  By default, it tries to detect the best
 factorisation, but you can force its output.
 You can adjust the number of decimals with the @code{SCALE}
 environment variable.")
-    (license license:wtfpl2))))
+    (license license:wtfpl2)))
 
 (define-public fortify-headers
   (package
     (name "fortify-headers")
-    (version "0.8")
+    (version "1.0")
     (source
      (origin
        (method url-fetch)
@@ -571,15 +567,15 @@ environment variable.")
                            name "-" version ".tar.gz"))
        (sha256
         (base32
-         "1cacdczpjb49c4i1168g541wnl3i3gbpv2m2wbnmw5wddlyhgkdg"))))
+         "0nvplfg4y4a10x2j12qwmxzlk2q4j6287j0v5f9bfsdayb04qvh2"))))
     (build-system gnu-build-system)
     (arguments
-     `(#:tests? #f ; No tests
+     `(#:tests? #f                      ; no tests
        #:make-flags (list "CC=gcc"
                           (string-append "PREFIX=" %output))
        #:phases
        (modify-phases %standard-phases
-         (delete 'configure)))) ; No configure script
+         (delete 'configure))))         ; no configure script
     (home-page "https://git.2f30.org/fortify-headers/")
     (synopsis "Standalone fortify-source implementation")
     (description
@@ -681,8 +677,8 @@ as -1, to be used instead of U+FFFD.
 
 ;; No release tarballs so far.
 (define-public lchat
-  (let ((revision "2")
-        (commit "25d90f4630b45e2b609d2e3daecb32cf5ff065fd"))
+  (let ((revision "3")
+        (commit "f95191970fd59c52a8b09cff32bd8d2135cbfc6b"))
     (package
       (name "lchat")
       (version (string-append "0.0.0-" revision "." (string-take commit 7)))
@@ -695,24 +691,26 @@ as -1, to be used instead of U+FFFD.
          (file-name (string-append name "-" version "-checkout"))
          (sha256
           (base32
-           "0dvljyq3m7rxxkqv7rkmijak6vj8i4db3iq2z988bvf76chz268b"))))
+           "07pxzziczhzprmjy61k7nl9i1kxpgnad37qkjf5fn4wf06nqdxpl"))))
       (build-system gnu-build-system)
       (arguments
-       `(#:tests? #f ; No tests
+       `(#:test-target "test"
          #:make-flags (list "CC=gcc"
                             (string-append "PREFIX=" %output))
          #:phases
          (modify-phases %standard-phases
-           (delete 'configure) ; No configure script
+           (delete 'configure)          ; no configure script
            (add-before 'build 'libbsd
              (lambda _
                (substitute* "Makefile"
                  (("-lutf") "-lutf -lbsd"))))
            (replace 'install
              (lambda* (#:key outputs #:allow-other-keys)
-               (let* ((out (assoc-ref outputs "out"))
-                      (bin (string-append out "/bin")))
+               (let* ((out  (assoc-ref outputs "out"))
+                      (bin  (string-append out "/bin"))
+                      (man1 (string-append out "/share/man/man1")))
                  (install-file "lchat" bin)
+                 (install-file "lchat.1" man1)
                  #t))))))
       (inputs
        `(("grep" ,grep)
@@ -723,6 +721,34 @@ as -1, to be used instead of U+FFFD.
       (synopsis "Line chat is a frontend for the irc client ii from suckless")
       (description
        "Lchat (line chat) is the little and small brother of cii.
-It is a front end for ii-like chat programs.  It uses tail(1) -f to get the
-chat output in background.")
+It is a front end for ii-like chat programs.  It uses @code{tail -f} to get the
+chat output in the background.")
       (license license:isc))))
+
+(define-public scron
+  (package
+    (name "scron")
+    (version "0.4")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "https://dl.2f30.org/releases/"
+                           name "-" version ".tar.gz"))
+       (sha256
+        (base32
+         "066fwa55kqcgfrsqgxh94sqbkxfsr691360xg4ljxr4i75d25s2a"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:tests? #f ; No tests
+       #:make-flags (list "CC=gcc"
+                          (string-append "PREFIX=" %output))
+       #:phases
+       (modify-phases %standard-phases
+         (delete 'configure)))) ; No configure script
+    (home-page "https://git.2f30.org/scron/")
+    (synopsis "Simple cron daemon")
+    (description
+     "Schedule commands to be run at specified dates and times.
+Single daemon and configuration file.  Log to stdout or syslog.  No mail
+support.")
+    (license license:expat)))
