@@ -1,9 +1,11 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2013, 2014, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2013, 2014, 2015 Andreas Enge <andreas@enge.fr>
 ;;; Copyright © 2015 Federico Beffa <beffa@fbengineering.ch>
 ;;; Copyright © 2016 Alex Griffin <a@ajgrf.com>
 ;;; Copyright © 2016 David Craven <david@craven.ch>
+;;; Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2019 Guy Fleury Iteriteka <hoonandon@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -34,7 +36,9 @@
   #:use-module (srfi srfi-26)
   #:export (%mirrors
             url-fetch
+            url-fetch/executable
             url-fetch/tarbomb
+            url-fetch/zipbomb
             download-to-store))
 
 ;;; Commentary:
@@ -48,7 +52,7 @@
   (let* ((gnu-mirrors
           '(;; This one redirects to a (supposedly) nearby and (supposedly)
             ;; up-to-date mirror.
-            "http://ftpmirror.gnu.org/"
+            "https://ftpmirror.gnu.org/gnu/"
 
             "ftp://ftp.cs.tu-berlin.de/pub/gnu/"
             "ftp://ftp.funet.fi/pub/mirrors/ftp.gnu.org/gnu/"
@@ -64,13 +68,13 @@
        "ftp://gcc.gnu.org/pub/gcc/"
        ,@(map (cut string-append <> "/gcc") gnu-mirrors))
       (gnupg
-       "ftp://gd.tuwien.ac.at/privacy/gnupg/"
+       "http://artfiles.org/gnupg.org"
+       "http://www.crysys.hu/"
+       "https://gnupg.org/ftp/gcrypt/"
        "ftp://mirrors.dotsrc.org/gcrypt/"
        "ftp://mirror.cict.fr/gnupg/"
-       "http://artfiles.org/gnupg.org"
        "ftp://ftp.franken.de/pub/crypt/mirror/ftp.gnupg.org/gcrypt/"
        "ftp://ftp.freenet.de/pub/ftp.gnupg.org/gcrypt/"
-       "http://www.crysys.hu/"
        "ftp://ftp.hi.is/pub/mirrors/gnupg/"
        "ftp://ftp.heanet.ie/mirrors/ftp.gnupg.org/gcrypt/"
        "ftp://ftp.bit.nl/mirror/gnupg/"
@@ -86,22 +90,23 @@
        "http://ftp.belnet.be/ftp.gnome.org/"
        "http://ftp.linux.org.uk/mirrors/ftp.gnome.org/"
        "http://ftp.gnome.org/pub/GNOME/"
+       "https://download.gnome.org/"
        "http://mirror.yandex.ru/mirrors/ftp.gnome.org/")
       (hackage
        "http://hackage.haskell.org/")
       (savannah
        "http://download.savannah.gnu.org/releases/"
-       "ftp://ftp.twaren.net/Unix/NonGNU/"
-       "ftp://mirror.csclub.uwaterloo.ca/nongnu/"
-       "ftp://mirror.publicns.net/pub/nongnu/"
-       "ftp://savannah.c3sl.ufpr.br/"
        "http://ftp.cc.uoc.gr/mirrors/nongnu.org/"
        "http://ftp.twaren.net/Unix/NonGNU/"
        "http://mirror.csclub.uwaterloo.ca/nongnu/"
        "http://nongnu.askapache.com/"
        "http://savannah.c3sl.ufpr.br/"
-       "http://www.centervenus.com/mirrors/nongnu/"
-       "http://download.savannah.gnu.org/releases-noredirect/")
+       "http://download.savannah.gnu.org/releases-noredirect/"
+       "http://download-mirror.savannah.gnu.org/releases/"
+       "ftp://ftp.twaren.net/Unix/NonGNU/"
+       "ftp://mirror.csclub.uwaterloo.ca/nongnu/"
+       "ftp://mirror.publicns.net/pub/nongnu/"
+       "ftp://savannah.c3sl.ufpr.br/")
       (sourceforge ; https://sourceforge.net/p/forge/documentation/Mirrors/
        "http://downloads.sourceforge.net/project/"
        "http://ufpr.dl.sourceforge.net/project/"
@@ -111,7 +116,7 @@
        "http://jaist.dl.sourceforge.net/project/"
        "http://kent.dl.sourceforge.net/project/"
        "http://liquidtelecom.dl.sourceforge.net/project/"
-       "http://nbtelecom.dl.sourceforge.net/project/"
+       ;; "http://nbtelecom.dl.sourceforge.net/project/"  ;never returns 404s
        "http://nchc.dl.sourceforge.net/project/"
        "http://ncu.dl.sourceforge.net/project/"
        "http://netcologne.dl.sourceforge.net/project/"
@@ -121,18 +126,20 @@
        "http://tenet.dl.sourceforge.net/project/"
        "http://vorboss.dl.sourceforge.net/project/"
        "http://netassist.dl.sourceforge.net/project/")
+      (netfilter.org ; https://www.netfilter.org/mirrors.html
+       "http://ftp.netfilter.org/pub/"
+       "ftp://ftp.es.netfilter.org/mirrors/netfilter/"
+       "ftp://ftp.hu.netfilter.org/"
+       "ftp://www.lt.netfilter.org/pub/")
       (kernel.org
-       "http://www.all.kernel.org/pub/"
-       "http://ramses.wh2.tu-dresden.de/pub/mirrors/kernel.org/"
        "http://linux-kernel.uio.no/pub/"
        "http://kernel.osuosl.org/pub/"
-       "ftp://ftp.funet.fi/pub/mirrors/ftp.kernel.org/pub/"
        "http://ftp.be.debian.org/pub/"
-       "http://mirror.linux.org.au/")
+       "http://mirror.linux.org.au/"
+       "ftp://ftp.funet.fi/pub/mirrors/ftp.kernel.org/pub/")
       (apache             ; from http://www.apache.org/mirrors/dist.html
        "http://www.eu.apache.org/dist/"
        "http://www.us.apache.org/dist/"
-       "ftp://gd.tuwien.ac.at/pub/infosys/servers/http/apache/dist/"
        "http://apache.belnet.be/"
        "http://mirrors.ircam.fr/pub/apache/"
        "http://apache-mirror.rbc.ru/pub/apache/"
@@ -141,11 +148,11 @@
        "http://archive.apache.org/dist/")
       (xorg               ; from http://www.x.org/wiki/Releases/Download
        "http://www.x.org/releases/" ; main mirrors
-       "ftp://mirror.csclub.uwaterloo.ca/x.org/" ; North America
-       "ftp://xorg.mirrors.pair.com/"
-       "http://mirror.csclub.uwaterloo.ca/x.org/"
+       "http://mirror.csclub.uwaterloo.ca/x.org/" ; North America
        "http://xorg.mirrors.pair.com/"
        "http://mirror.us.leaseweb.net/xorg/"
+       "ftp://mirror.csclub.uwaterloo.ca/x.org/"
+       "ftp://xorg.mirrors.pair.com/"
        "ftp://artfiles.org/x.org/" ; Europe
        "ftp://ftp.chg.ru/pub/X11/x.org/"
        "ftp://ftp.fu-berlin.de/unix/X11/FTP.X.ORG/"
@@ -155,41 +162,74 @@
        "ftp://ftp.piotrkosoft.net/pub/mirrors/ftp.x.org/"
        "ftp://ftp.portal-to-web.de/pub/mirrors/x.org/"
        "ftp://ftp.solnet.ch/mirror/x.org/"
-       "ftp://gd.tuwien.ac.at/X11/"
        "ftp://mi.mirror.garr.it/mirrors/x.org/"
        "ftp://mirror.cict.fr/x.org/"
        "ftp://mirror.switch.ch/mirror/X11/"
        "ftp://mirrors.ircam.fr/pub/x.org/"
        "ftp://x.mirrors.skynet.be/pub/ftp.x.org/"
-       "ftp://ftp.cs.cuhk.edu.hk/pub/X11" ; East Asia
+       "http://x.cs.pu.edu.tw/" ; East Asia
+       "ftp://ftp.cs.cuhk.edu.hk/pub/X11"
        "ftp://ftp.u-aizu.ac.jp/pub/x11/x.org/"
        "ftp://ftp.yz.yamagata-u.ac.jp/pub/X11/x.org/"
        "ftp://ftp.kaist.ac.kr/x.org/"
        "ftp://mirrors.go-part.com/xorg/"
-       "http://x.cs.pu.edu.tw/"
        "ftp://ftp.is.co.za/pub/x.org")            ; South Africa
-      (cpan                              ; from http://www.cpan.org/SITES.html
-       "http://mirror.ibcp.fr/pub/CPAN/"
-       "ftp://ftp.ciril.fr/pub/cpan/"
-       "ftp://artfiles.org/cpan.org/"
+      (cpan
        "http://www.cpan.org/"
-       "ftp://cpan.rinet.ru/pub/mirror/CPAN/"
-       "ftp://cpan.inode.at/"
-       "ftp://cpan.iht.co.il/"
-       "ftp://ftp.osuosl.org/pub/CPAN/"
-       "ftp://ftp.nara.wide.ad.jp/pub/CPAN/"
-       "http://mirrors.163.com/cpan/"
-       "ftp://cpan.mirror.ac.za/"
-       "http://cpan.mirrors.ionfish.org/"
-       "http://cpan.mirror.dkm.cz/pub/CPAN/"
-       "http://cpan.mirror.iphh.net/"
-       "http://mirrors.teentelecom.net/CPAN/"
-       "http://mirror.teklinks.com/CPAN/"
-       "http://cpan.weepeetelecom.be/"
-       "http://mirrors.xservers.ro/CPAN/"
-       "http://cpan.yimg.com/"
-       "http://mirror.yazd.ac.ir/cpan/"
-       "http://ftp.belnet.be/ftp.cpan.org/")
+       "http://cpan.metacpan.org/"
+       ;; A selection of HTTP mirrors from http://www.cpan.org/SITES.html.
+       ;; Europe.
+       "http://ftp.belnet.be/mirror/ftp.cpan.org/"
+       "http://mirrors.nic.cz/CPAN/"
+       "http://mirror.ibcp.fr/pub/CPAN/"
+       "http://ftp.ntua.gr/pub/lang/perl/"
+       "http://mirror.as43289.net/pub/CPAN/"
+       "http://cpan.cs.uu.nl/"
+       "http://cpan.uib.no/"
+       "http://cpan-mirror.rbc.ru/pub/CPAN/"
+       "http://mirror.sbb.rs/CPAN/"
+       "http://cpan.lnx.sk/"
+       "http://ftp.rediris.es/mirror/CPAN/"
+       "http://mirror.ox.ac.uk/sites/www.cpan.org/"
+       ;; Africa.
+       "http://mirror.liquidtelecom.com/CPAN/"
+       "http://cpan.mirror.ac.za/"
+       "http://mirror.is.co.za/pub/cpan/"
+       "http://cpan.saix.net/"
+       "http://mirror.ucu.ac.ug/cpan/"
+       ;; North America.
+       "http://mirrors.gossamer-threads.com/CPAN/"
+       "http://mirror.csclub.uwaterloo.ca/CPAN/"
+       "http://mirrors.ucr.ac.cr/CPAN/"
+       "http://www.msg.com.mx/CPAN/"
+       "http://mirrors.namecheap.com/CPAN/"
+       "http://mirror.uic.edu/CPAN/"
+       "http://mirror.datapipe.net/CPAN/"
+       "http://mirror.cc.columbia.edu/pub/software/cpan/"
+       "http://mirror.uta.edu/CPAN/"
+       ;; South America.
+       "http://cpan.mmgdesigns.com.ar/"
+       "http://mirror.nbtelecom.com.br/CPAN/"
+       "http://linorg.usp.br/CPAN/"
+       "http://cpan.dcc.uchile.cl/"
+       "http://mirror.cedia.org.ec/CPAN/"
+       ;; Oceania.
+       "http://cpan.mirror.serversaustralia.com.au/"
+       "http://mirror.waia.asn.au/pub/cpan/"
+       "http://mirror.as24220.net/pub/cpan/"
+       "http://cpan.lagoon.nc/pub/CPAN/"
+       "http://cpan.inspire.net.nz/"
+       ;; Asia.
+       "http://mirror.dhakacom.com/CPAN/"
+       "http://mirrors.ustc.edu.cn/CPAN/"
+       "http://ftp.cuhk.edu.hk/pub/packages/perl/CPAN/"
+       "http://kambing.ui.ac.id/cpan/"
+       "http://cpan.hostiran.ir/"
+       "http://ftp.nara.wide.ad.jp/pub/CPAN/"
+       "http://mirror.neolabs.kz/CPAN/"
+       "http://cpan.nctu.edu.tw/"
+       "http://cpan.ulak.net.tr/"
+       "http://mirrors.vinahost.vn/CPAN/")
       (cran
        ;; Arbitrary mirrors from http://cran.r-project.org/mirrors.html
        ;; This one automatically redirects to servers worldwide
@@ -204,23 +244,20 @@
        ;; from http://www.imagemagick.org/script/download.php
        ;; (without mirrors that are unavailable or not up to date)
        ;; mirrors keeping old versions at the top level
-       "ftp://sunsite.icm.edu.pl/packages/ImageMagick/"
+       "https://sunsite.icm.edu.pl/packages/ImageMagick/"
        ;; mirrors moving old versions to "legacy"
-       "http://mirrors-au.go-parts.com/mirrors/ImageMagick/"
-       "ftp://mirror.aarnet.edu.au/pub/imagemagick/"
+       "http://mirrors-usa.go-parts.com/mirrors/ImageMagick/"
        "http://mirror.checkdomain.de/imagemagick/"
+       "http://ftp.surfnet.nl/pub/ImageMagick/"
+       "http://mirror.searchdaimon.com/ImageMagick"
+       "http://mirror.is.co.za/pub/imagemagick/"
+       "http://www.imagemagick.org/download/"
+       "ftp://mirror.aarnet.edu.au/pub/imagemagick/"
        "ftp://ftp.kddlabs.co.jp/graphics/ImageMagick/"
        "ftp://ftp.u-aizu.ac.jp/pub/graphics/image/ImageMagick/imagemagick.org/"
        "ftp://ftp.nluug.nl/pub/ImageMagick/"
-       "http://ftp.surfnet.nl/pub/ImageMagick/"
-       "http://mirror.searchdaimon.com/ImageMagick"
        "ftp://ftp.tpnet.pl/pub/graphics/ImageMagick/"
-       "http://mirrors-ru.go-parts.com/mirrors/ImageMagick/"
-       "http://mirror.is.co.za/pub/imagemagick/"
-       "http://mirrors-uk.go-parts.com/mirrors/ImageMagick/"
-       "http://mirrors-usa.go-parts.com/mirrors/ImageMagick/"
        "ftp://ftp.fifi.org/pub/ImageMagick/"
-       "http://www.imagemagick.org/download/"
        ;; one legacy location as a last resort
        "http://www.imagemagick.org/download/legacy/")
       (debian
@@ -230,6 +267,7 @@
        "http://archive.debian.org/debian/")
       (kde
        "http://download.kde.org"
+       "http://download.kde.org/Attic" ; for when it gets archived.
        ;; Mirrors from http://files.kde.org/extra/mirrors.html
        ;; Europe
        "http://mirror.easyname.at/kde"
@@ -273,7 +311,59 @@
        "http://kde.mirrors.tds.net/pub/kde/"
        ;; Oceania
        "http://ftp.kddlabs.co.jp/pub/X11/kde/"
-       "http://kde.mirror.uber.com.au/"))))
+       "http://kde.mirror.uber.com.au/")
+      (openbsd
+       "https://ftp.openbsd.org/pub/OpenBSD/"
+       ;; Anycast CDN redirecting to your friendly local mirror.
+       "https://mirrors.evowise.com/pub/OpenBSD/"
+       ;; Other HTTPS mirrors from https://www.openbsd.org/ftp.html
+       "https://mirror.aarnet.edu.au/pub/OpenBSD/"
+       "https://ftp2.eu.openbsd.org/pub/OpenBSD/"
+       "https://openbsd.c3sl.ufpr.br/pub/OpenBSD/"
+       "https://openbsd.ipacct.com/pub/OpenBSD/"
+       "https://ftp.OpenBSD.org/pub/OpenBSD/"
+       "https://openbsd.cs.toronto.edu/pub/OpenBSD/"
+       "https://openbsd.delfic.org/pub/OpenBSD/"
+       "https://openbsd.mirror.netelligent.ca/pub/OpenBSD/"
+       "https://mirrors.ucr.ac.cr/pub/OpenBSD/"
+       "https://mirrors.dotsrc.org/pub/OpenBSD/"
+       "https://mirror.one.com/pub/OpenBSD/"
+       "https://ftp.fr.openbsd.org/pub/OpenBSD/"
+       "https://ftp2.fr.openbsd.org/pub/OpenBSD/"
+       "https://mirrors.ircam.fr/pub/OpenBSD/"
+       "https://ftp.spline.de/pub/OpenBSD/"
+       "https://mirror.hs-esslingen.de/pub/OpenBSD/"
+       "https://ftp.halifax.rwth-aachen.de/openbsd/"
+       "https://ftp.hostserver.de/pub/OpenBSD/"
+       "https://ftp.fau.de/pub/OpenBSD/"
+       "https://ftp.cc.uoc.gr/pub/OpenBSD/"
+       "https://openbsd.hk/pub/OpenBSD/"
+       "https://ftp.heanet.ie/pub/OpenBSD/"
+       "https://openbsd.mirror.garr.it/pub/OpenBSD/"
+       "https://mirror.litnet.lt/pub/OpenBSD/"
+       "https://mirror.meerval.net/pub/OpenBSD/"
+       "https://ftp.nluug.nl/pub/OpenBSD/"
+       "https://ftp.bit.nl/pub/OpenBSD/"
+       "https://mirrors.dalenys.com/pub/OpenBSD/"
+       "https://ftp.icm.edu.pl/pub/OpenBSD/"
+       "https://ftp.rnl.tecnico.ulisboa.pt/pub/OpenBSD/"
+       "https://mirrors.pidginhost.com/pub/OpenBSD/"
+       "https://mirror.yandex.ru/pub/OpenBSD/"
+       "https://ftp.eu.openbsd.org/pub/OpenBSD/"
+       "https://ftp.yzu.edu.tw/pub/OpenBSD/"
+       "https://www.mirrorservice.org/pub/OpenBSD/"
+       "https://anorien.csc.warwick.ac.uk/pub/OpenBSD/"
+       "https://mirror.bytemark.co.uk/pub/OpenBSD/"
+       "https://mirrors.sonic.net/pub/OpenBSD/"
+       "https://ftp3.usa.openbsd.org/pub/OpenBSD/"
+       "https://mirrors.syringanetworks.net/pub/OpenBSD/"
+       "https://openbsd.mirror.constant.com/pub/OpenBSD/"
+       "https://ftp4.usa.openbsd.org/pub/OpenBSD/"
+       "https://ftp5.usa.openbsd.org/pub/OpenBSD/"
+       "https://mirror.esc7.net/pub/OpenBSD/")
+      (mate
+       "https://pub.mate-desktop.org/releases/"
+       "http://pub.mate-desktop.org/releases/"))))
 
 (define %mirror-file
   ;; Copy of the list of mirrors to a file.  This allows us to keep a single
@@ -285,48 +375,53 @@
   ;; List of content-addressed mirrors.  Each mirror is represented as a
   ;; procedure that takes a file name, an algorithm (symbol) and a hash
   ;; (bytevector), and returns a URL or #f.
-  ;; Note: Avoid 'https' to mitigate <http://bugs.gnu.org/22774>.
-  ;; TODO: Add more.
-  '(list (lambda (file algo hash)
-           ;; Files served by 'guix publish' are accessible under a single
-           ;; hash algorithm.
-           (string-append "http://mirror.hydra.gnu.org/file/"
-                          file "/" (symbol->string algo) "/"
-                          (bytevector->nix-base32-string hash)))
-         (lambda (file algo hash)
-           ;; 'tarballs.nixos.org' supports several algorithms.
-           (string-append "http://tarballs.nixos.org/"
-                          (symbol->string algo) "/"
-                          (bytevector->nix-base32-string hash)))))
+  '(begin
+     (use-modules (guix base32))
+
+     (define (guix-publish host)
+       (lambda (file algo hash)
+         ;; Files served by 'guix publish' are accessible under a single
+         ;; hash algorithm.
+         (string-append "https://" host "/file/"
+                        file "/" (symbol->string algo) "/"
+                        (bytevector->nix-base32-string hash))))
+
+     ;; XXX: (guix base16) appeared in March 2017 (and thus 0.13.0) so old
+     ;; installations of the daemon might lack it.  Thus, load it lazily to
+     ;; avoid gratuitous errors.  See <https://bugs.gnu.org/33542>.
+     (module-autoload! (current-module)
+                       '(guix base16) '(bytevector->base16-string))
+
+     (list (guix-publish "ci.guix.gnu.org")
+           (lambda (file algo hash)
+             ;; 'tarballs.nixos.org' supports several algorithms.
+             (string-append "https://tarballs.nixos.org/"
+                            (symbol->string algo) "/"
+                            (bytevector->nix-base32-string hash)))
+           (lambda (file algo hash)
+             ;; Software Heritage usually archives VCS history rather than
+             ;; tarballs, but tarballs are sometimes available (and can be
+             ;; explicitly stored there.)  For example, see
+             ;; <https://archive.softwareheritage.org/api/1/content/sha256:92d0fa1c311cacefa89853bdb53c62f4110cdfda3820346b59cbd098f40f955e/>.
+             (string-append "https://archive.softwareheritage.org/api/1/content/"
+                            (symbol->string algo) ":"
+                            (bytevector->base16-string hash) "/raw/")))))
 
 (define %content-addressed-mirror-file
   ;; Content-addressed mirrors stored in a file.
   (plain-file "content-addressed-mirrors"
               (object->string %content-addressed-mirrors)))
 
-(define (gnutls-package)
-  "Return the default GnuTLS package."
-  (let ((module (resolve-interface '(gnu packages tls))))
-    (module-ref module 'gnutls)))
-
 (define built-in-builders*
-  (let ((cache (make-weak-key-hash-table)))
-    (lambda ()
-      "Return, as a monadic value, the list of built-in builders supported by
-the daemon."
-      (lambda (store)
-        ;; Memoize the result to avoid repeated RPCs.
-        (values (or (hashq-ref cache store)
-                    (let ((result (built-in-builders store)))
-                      (hashq-set! cache store result)
-                      result))
-                store)))))
+  (store-lift built-in-builders))
 
 (define* (built-in-download file-name url
                             #:key system hash-algo hash
                             mirrors content-addressed-mirrors
+                            executable?
                             (guile 'unused))
-  "Download FILE-NAME from URL using the built-in 'download' builder.
+  "Download FILE-NAME from URL using the built-in 'download' builder.  When
+EXECUTABLE? is true, make the downloaded file executable.
 
 This is an \"out-of-band\" download in that the returned derivation does not
 explicitly depend on Guile, GnuTLS, etc.  Instead, the daemon performs the
@@ -338,8 +433,8 @@ download by itself using its own dependencies."
                     #:system system
                     #:hash-algo hash-algo
                     #:hash hash
-                    #:inputs `((,mirrors)
-                               (,content-addressed-mirrors))
+                    #:recursive? executable?
+                    #:sources (list mirrors content-addressed-mirrors)
 
                     ;; Honor the user's proxy and locale settings.
                     #:leaked-env-vars '("http_proxy" "https_proxy"
@@ -349,7 +444,10 @@ download by itself using its own dependencies."
                     #:env-vars `(("url" . ,(object->string url))
                                  ("mirrors" . ,mirrors)
                                  ("content-addressed-mirrors"
-                                  . ,content-addressed-mirrors))
+                                  . ,content-addressed-mirrors)
+                                 ,@(if executable?
+                                       '(("executable" . "1"))
+                                       '()))
 
                     ;; Do not offload this derivation because we cannot be
                     ;; sure that the remote daemon supports the 'download'
@@ -357,97 +455,16 @@ download by itself using its own dependencies."
                     ;; for that built-in is widespread.
                     #:local-build? #t)))
 
-(define* (in-band-download file-name url
-                           #:key system hash-algo hash
-                           mirrors content-addressed-mirrors
-                           guile)
-  "Download FILE-NAME from URL using a normal, \"in-band\" fixed-output
-derivation.
-
-This is now deprecated since it has the drawback of causing bootstrapping
-issues: we may need to build GnuTLS just to be able to download the source of
-GnuTLS itself and its dependencies.  See <http://bugs.gnu.org/22774>."
-  (define need-gnutls?
-    ;; True if any of the URLs need TLS support.
-    (let ((https? (cut string-prefix? "https://" <>)))
-      (match url
-        ((? string?)
-         (https? url))
-        ((url ...)
-         (any https? url)))))
-
-  (define builder
-    (with-imported-modules '((guix build download)
-                             (guix build utils)
-                             (guix ftp-client)
-                             (guix base32)
-                             (guix base64))
-      #~(begin
-          #+(if need-gnutls?
-
-                ;; Add GnuTLS to the inputs and to the load path.
-                #~(eval-when (load expand eval)
-                    (set! %load-path
-                      (cons (string-append #+(gnutls-package)
-                                           "/share/guile/site/"
-                                           (effective-version))
-                            %load-path)))
-                #~#t)
-
-          (use-modules (guix build download)
-                       (guix base32))
-
-          (let ((value-from-environment (lambda (variable)
-                                          (call-with-input-string
-                                              (getenv variable)
-                                            read))))
-            (url-fetch (value-from-environment "guix download url")
-                       #$output
-                       #:mirrors (call-with-input-file #$mirrors read)
-
-                       ;; Content-addressed mirrors.
-                       #:hashes
-                       (value-from-environment "guix download hashes")
-                       #:content-addressed-mirrors
-                       (primitive-load #$content-addressed-mirrors)
-
-                       ;; No need to validate certificates since we know the
-                       ;; hash of the expected result.
-                       #:verify-certificate? #f)))))
-
-  (mlet %store-monad ((guile (package->derivation guile system)))
-    (gexp->derivation file-name builder
-                      #:guile-for-build guile
-                      #:system system
-                      #:hash-algo hash-algo
-                      #:hash hash
-
-                      ;; Use environment variables and a fixed script
-                      ;; name so there's only one script in store for
-                      ;; all the downloads.
-                      #:script-name "download"
-                      #:env-vars
-                      `(("guix download url" . ,(object->string url))
-                        ("guix download hashes"
-                         . ,(object->string `((,hash-algo . ,hash)))))
-
-                      ;; Honor the user's proxy settings.
-                      #:leaked-env-vars '("http_proxy" "https_proxy")
-
-                      ;; In general, offloading downloads is not a good
-                      ;; idea.  Daemons before 0.8.3 would also
-                      ;; interpret this as "do not substitute" (see
-                      ;; <https://bugs.gnu.org/18747>.)
-                      #:local-build? #t)))
-
 (define* (url-fetch url hash-algo hash
                     #:optional name
                     #:key (system (%current-system))
-                    (guile (default-guile)))
+                    (guile (default-guile))
+                    executable?)
   "Return a fixed-output derivation that fetches URL (a string, or a list of
 strings denoting alternate URLs), which is expected to have hash HASH of type
 HASH-ALGO (a symbol).  By default, the file name is the base name of URL;
-optionally, NAME can specify a different file name.
+optionally, NAME can specify a different file name.  When EXECUTABLE? is true,
+make the downloaded file executable.
 
 When one of the URL starts with mirror://, then its host part is
 interpreted as the name of a mirror scheme, taken from %MIRROR-FILE.
@@ -466,18 +483,32 @@ in the store."
             (and uri (memq (uri-scheme uri) '(#f file))))
         (interned-file (if uri (uri-path uri) url)
                        (or name file-name))
-        (mlet* %store-monad ((builtins (built-in-builders*))
-                             (download -> (if (member "download" builtins)
-                                              built-in-download
-                                              in-band-download)))
-          (download (or name file-name) url
-                    #:guile guile
-                    #:system system
-                    #:hash-algo hash-algo
-                    #:hash hash
-                    #:mirrors %mirror-file
-                    #:content-addressed-mirrors
-                    %content-addressed-mirror-file)))))
+        (mlet %store-monad ((builtins (built-in-builders*)))
+          ;; The "download" built-in builder was added in guix-daemon in
+          ;; Nov. 2016 and made it in the 0.12.0 release of Dec. 2016.  We now
+          ;; require it.
+          (unless (member "download" builtins)
+            (error "'guix-daemon' is too old, please upgrade" builtins))
+
+          (built-in-download (or name file-name) url
+                             #:guile guile
+                             #:system system
+                             #:hash-algo hash-algo
+                             #:hash hash
+                             #:executable? executable?
+                             #:mirrors %mirror-file
+                             #:content-addressed-mirrors
+                             %content-addressed-mirror-file)))))
+
+(define* (url-fetch/executable url hash-algo hash
+                               #:optional name
+                               #:key (system (%current-system))
+                               (guile (default-guile)))
+  "Like 'url-fetch', but make the downloaded file executable."
+  (url-fetch url hash-algo hash name
+             #:system system
+             #:guile guile
+             #:executable? #t))
 
 (define* (url-fetch/tarbomb url hash-algo hash
                             #:optional name
@@ -485,23 +516,75 @@ in the store."
                             (guile (default-guile)))
   "Similar to 'url-fetch' but unpack the file from URL in a directory of its
 own.  This helper makes it easier to deal with \"tar bombs\"."
+  (define file-name
+    (match url
+      ((head _ ...)
+       (basename head))
+      (_
+       (basename url))))
   (define gzip
     (module-ref (resolve-interface '(gnu packages compression)) 'gzip))
   (define tar
     (module-ref (resolve-interface '(gnu packages base)) 'tar))
 
   (mlet %store-monad ((drv (url-fetch url hash-algo hash
-                                      (string-append "tarbomb-" name)
+                                      (string-append "tarbomb-"
+                                                     (or name file-name))
                                       #:system system
-                                      #:guile guile)))
+                                      #:guile guile))
+                      (guile (package->derivation guile system)))
     ;; Take the tar bomb, and simply unpack it as a directory.
-    (gexp->derivation name
-                      #~(begin
-                          (mkdir #$output)
-                          (setenv "PATH" (string-append #$gzip "/bin"))
-                          (chdir #$output)
-                          (zero? (system* (string-append #$tar "/bin/tar")
-                                          "xf" #$drv)))
+    ;; Use ungrafted tar/gzip so that the resulting tarball doesn't depend on
+    ;; whether grafts are enabled.
+    (gexp->derivation (or name file-name)
+                      (with-imported-modules '((guix build utils))
+                        #~(begin
+                            (use-modules (guix build utils))
+                            (mkdir #$output)
+                            (setenv "PATH" (string-append #$gzip "/bin"))
+                            (chdir #$output)
+                            (invoke (string-append #$tar "/bin/tar")
+                                    "xf" #$drv)))
+                      #:system system
+                      #:guile-for-build guile
+                      #:graft? #f
+                      #:local-build? #t)))
+
+(define* (url-fetch/zipbomb url hash-algo hash
+                            #:optional name
+                            #:key (system (%current-system))
+                            (guile (default-guile)))
+  "Similar to 'url-fetch' but unpack the zip file at URL in a directory of its
+own.  This helper makes it easier to deal with \"zip bombs\"."
+  (define file-name
+    (match url
+      ((head _ ...)
+       (basename head))
+      (_
+       (basename url))))
+  (define unzip
+    (module-ref (resolve-interface '(gnu packages compression)) 'unzip))
+
+  (mlet %store-monad ((drv (url-fetch url hash-algo hash
+                                      (string-append "zipbomb-"
+                                                     (or name file-name))
+                                      #:system system
+                                      #:guile guile))
+                      (guile (package->derivation guile system)))
+    ;; Take the zip bomb, and simply unpack it as a directory.
+    ;; Use ungrafted unzip so that the resulting tarball doesn't depend on
+    ;; whether grafts are enabled.
+    (gexp->derivation (or name file-name)
+                      (with-imported-modules '((guix build utils))
+                        #~(begin
+                            (use-modules (guix build utils))
+                            (mkdir #$output)
+                            (chdir #$output)
+                            (invoke (string-append #$unzip "/bin/unzip")
+                                    #$drv)))
+                      #:system system
+                      #:guile-for-build guile
+                      #:graft? #f
                       #:local-build? #t)))
 
 (define* (download-to-store store url #:optional (name (basename url))
