@@ -1,6 +1,8 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2012, 2013, 2014 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2014 David Thompson <davet@gnu.org>
+;;; Copyright © 2018 Kyle Meyer <kyle@kyleam.com>
+;;; Copyright © 2019 Ricardo Wurmus <rekado@elephly.net>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -73,7 +75,8 @@ rather than \\n."
 ;;; Entry point.
 ;;;
 
-(define importers '("gnu" "nix" "pypi" "cpan" "hackage" "elpa" "gem" "cran" "crate"))
+(define importers '("gnu" "nix" "pypi" "cpan" "hackage" "stackage" "elpa" "gem"
+                    "cran" "crate" "texlive" "json" "opam"))
 
 (define (resolve-importer name)
   (let ((module (resolve-interface
@@ -82,15 +85,15 @@ rather than \\n."
     (module-ref module proc)))
 
 (define (show-help)
-  (display (_ "Usage: guix import IMPORTER ARGS ...
+  (display (G_ "Usage: guix import IMPORTER ARGS ...
 Run IMPORTER with ARGS.\n"))
   (newline)
-  (display (_ "IMPORTER must be one of the importers listed below:\n"))
+  (display (G_ "IMPORTER must be one of the importers listed below:\n"))
   (newline)
   (format #t "~{   ~a~%~}" importers)
-  (display (_ "
+  (display (G_ "
   -h, --help             display this help and exit"))
-  (display (_ "
+  (display (G_ "
   -V, --version          display version information and exit"))
   (newline)
   (show-bug-report-information))
@@ -99,11 +102,11 @@ Run IMPORTER with ARGS.\n"))
   (match args
     (()
      (format (current-error-port)
-             (_ "guix import: missing importer name~%")))
+             (G_ "guix import: missing importer name~%")))
     ((or ("-h") ("--help"))
      (show-help)
      (exit 0))
-    (("--version")
+    ((or ("-V") ("--version"))
      (show-version-and-exit "guix import"))
     ((importer args ...)
      (if (member importer importers)
@@ -111,7 +114,8 @@ Run IMPORTER with ARGS.\n"))
                         (pretty-print expr (newline-rewriting-port
                                             (current-output-port))))))
            (match (apply (resolve-importer importer) args)
-             ((and expr ('package _ ...))
+             ((and expr (or ('package _ ...)
+                            ('let _ ...)))
               (print expr))
              ((? list? expressions)
               (for-each (lambda (expr)
@@ -119,5 +123,5 @@ Run IMPORTER with ARGS.\n"))
                           (newline))
                         expressions))
              (x
-              (leave (_ "'~a' import failed~%") importer))))
-         (leave (_ "~a: invalid importer~%") importer)))))
+              (leave (G_ "'~a' import failed~%") importer))))
+         (leave (G_ "~a: invalid importer~%") importer)))))
